package at.ac.uibk.gitsearch.repository.search.testESService;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Collections;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.common.settings.Settings;
import org.elasticsearch.node.InternalSettingsPreparer;
import org.elasticsearch.node.Node;
import org.elasticsearch.node.NodeValidationException;
import org.elasticsearch.plugins.Plugin;
import org.elasticsearch.transport.Netty4Plugin;
import org.springframework.boot.context.event.ApplicationPreparedEvent;
import org.springframework.boot.context.event.ApplicationStartingEvent;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.event.ContextStartedEvent;
import org.springframework.context.event.ContextStoppedEvent;
import org.springframework.context.event.EventListener;
import org.springframework.data.elasticsearch.client.NodeClientFactoryBean;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;

import com.google.common.base.Charsets;
import com.google.common.io.ByteSource;

// @Configuration not working, everything is final :-(
public class ElasticSearchTestServerConfiguration {
	
	private static final Logger LOGGER = LogManager.getLogger(ElasticSearchTestServerConfiguration.class);
    private TestESNode testNode;

    public ElasticSearchTestServerConfiguration() throws IOException, NodeValidationException {
    }

	public void startTestNode() throws IOException, NodeValidationException {
		File f = File.createTempFile("tobedeleted", "txt");
		Settings s =  Settings.builder().put("http.port", "29200")
		             .put("node.name", "esSharingTest")
                     .put("transport.type", "netty4")
	                 .put("http.type", "netty4")
		             .put("path.home", f.getParent() + File.pathSeparator + "esTest").build();
		
		NodeClientFactoryBean fb = new NodeClientFactoryBean(true);
		
		testNode = new TestESNode(s, Collections.singleton(Netty4Plugin.class)) {
			
		}; 
		testNode.start();
		
		setUpMetaDataIndex();
		setUpContent();
	}
	
	private void setUpMetaDataIndex() throws IOException {
		
		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
	    headers.setContentType(MediaType.APPLICATION_JSON);
	    
	    try {
			restTemplate.delete("http://localhost:29200/idx_metadata_1", String.class); // just in case, delete old index
		} catch (RestClientException e) {
			LOGGER.info("Deletion of previous index failed: {}!", e.getMessage());
		}
	    HttpEntity<String> request = 
	    	      new HttpEntity<String>(getMetaDataConfigDefinition(), headers);
	    
	    
	    restTemplate.put("http://localhost:29200/idx_metadata_1", request, String.class);
	    
	}
	
	private void setUpContent() throws IOException {
		

		RestTemplate restTemplate = new RestTemplate();
		HttpHeaders headers = new HttpHeaders();
	    headers.setContentType(MediaType.APPLICATION_JSON);
	    
	    int contentCount = 0;
	    
	    while(true) {
	    	contentCount++;
		    final String content = getContent(contentCount);
		    if(content==null) break;
			HttpEntity<String> request = 
		    	      new HttpEntity<String>(content, headers);
		    restTemplate.put("http://localhost:29200/metadata/_doc/"+contentCount, request, String.class);
	    }
	}
	private String getMetaDataConfigDefinition() throws IOException {
		InputStream metaDataStream = this.getClass().getResourceAsStream("/at/ac/uibk/gitsearch/repository/search/testESService/es_metadata.schema.json");
		ByteSource byteSource = new ByteSource() {
			
			@Override
			public InputStream openStream() throws IOException {
				return metaDataStream;
			}
		};
		String text = byteSource.asCharSource(Charsets.UTF_8).read();
		return text;
	}
	
	private String getContent(int i) throws IOException {
		InputStream metaDataStream = this.getClass().getResourceAsStream("/at/ac/uibk/gitsearch/repository/search/testESService/content"+i+".json");
		if(metaDataStream==null) return null;
		ByteSource byteSource = new ByteSource() {
			
			@Override
			public InputStream openStream() throws IOException {
				return metaDataStream;
			}
		};
		String text = byteSource.asCharSource(Charsets.UTF_8).read();
		return text;
	}

    
    @EventListener
    public void onApplicationEvent(ApplicationStartingEvent event) {
		try {
			startTestNode();
		} catch (IOException | NodeValidationException e) {
			LOGGER.error("Cannot startup TestNode", e);
		}
    }

	@EventListener
    public void onApplicationEvent(ContextStartedEvent event) {
		try {
			startTestNode();
		} catch (IOException | NodeValidationException e) {
			LOGGER.error("Cannot startup TestNode", e);
		}
    }
    
    @EventListener
    public void onApplicationEvent(ContextStoppedEvent event) {
		try {
			testNode.close();
		} catch (IOException  e) {
			LOGGER.error("Cannot startup TestNode", e);
		}
    }

    public static class TestESNode extends Node {
        public TestESNode(Settings preparedSettings, Collection<Class<? extends Plugin>> classpathPlugins) {
            super(InternalSettingsPreparer.prepareEnvironment(preparedSettings, null), classpathPlugins, true);
        }

		@Override
		protected void registerDerivedNodeNameWithLogger(String nodeName) {
			LogManager.getLogger(nodeName);
		}
}

    @Bean()
    public TestESNode getESTestServer() {
        return testNode;
    }
    
}
